;**********************************************************
;* File Name		:fz_03.asm
;* Title		:QRP-Zhler mit Preset
;* Version 		:1.3b/ 18.12.2002 
;* Version 		:1.3 / 14.12.2002 
;* Version 		:1.0 / 05.12.2002 Start 
;* 
;* Target MCU		:AT90S8515
;* 
;***********************************************************
;
.NOLIST
.include "../Include/8515def.inc"	; Proc Def's
.include "../Include/math_def.inc"	; Fr Math.inc

.list
.listmac

;* Konstanten 
;---------------------------------------------------------------------------

; Main Program Register variables
;---------------------------------------------------------------------------
.def	ac_tmp		= r9		; temporary storage register for SREG
.def 	temp 		= r16		; temp Byte
.def	byte		= r24		; Byte

; LCD - Ports und Defs 
; 4BIT-LCD (abhngig von CPU und Adapter, Variante A, HighByte)
;---------------------------------------------------------------------------
.equ	lcd_port	= PORTA		; 8515 PORTA , 1200/2313 PORTB
.equ	lcd_ddr		= DDRA		; 8515 DDRA  , 1200/2313 DDRB
.equ	lcd_rs		= 1		; Port 1 LCD Dispay control bits
.equ	lcd_en		= 3		; Port 3
.equ	lcd_rw		= 2		; Port 2
.equ	lcd_led		= 0		; Port 0, Beleuchtung ein
.def	lcd_pos		= r23		; Position 
.def	lcd_byte	= r24		; Auszugebendes Byte
.def 	lcd_temp	= r25		; diverses
.def 	lcd_tim1	= r26
.def 	lcd_tim2	= r27

; Zhler Ports & Constanten 
;---------------------------------------------------------------------------
.equ	CNT_PORT	= PORTD		; Port D
.equ	CNT_PIN		= PIND		; Pins Port D
.equ	CNT_DDR		= DDRD		; DataDirection D
.equ	CNT_CPORT	= PORTB		; Port B, Control register
.equ	CNT_CDDR	= DDRB		; DataDirection B
.equ	CNT_CLOCK	= 0		; PB0 , Counter Clock
.equ	CNT_ENABLE	= 1		; PB1 , Counter enable
.equ	CNT_CLEAR	= 2		; PB2 , Clear Impuls
.equ	CNT_CNT0	= 3		; PB4 , 1. Zhler
.equ	CNT_CNT1	= 4		; PB3 , 2. Zhler
.equ	CNT_CLED	= 7		; PB7 , TZ Led
.equ	CNT_PRE0	= 1		; Standard 1
.equ	CNT_PRE1	= 10		; Falls Vorteiler verwendet wird 1 :10
.equ	CNT_PRE2	= 64		; Falls Vorteiler verwendet wird 2 :64
.equ	CNT_SWITCH	= $0010		; Umschaltung Anzeige MHz/Hz bei 1048576 = $0010 0000
					; Es werden nur die beiden High-Bytes verglichen !

; Control Port (PortC) , default High 
;---------------------------------------------------------------------------
.equ	CTRL_PORT	= PORTC		; Port C
.equ	CTRL_PIN	= PINC		; Pins Port C
.equ	CTRL_DDR	= DDRC		; DataDirection C
.equ	CTRL_SW0	= 0		; PC0 , BCD-Switch 
.equ	CTRL_SW1	= 1		; PC1 , Offset 
.equ	CTRL_SW2	= 2		; PC2 , Einstellung
.equ	CTRL_SW4	= 3		; PC3 , sh. Tabelle
.equ	CTRL_PTT	= 4		; Offset Off , wenn Low
.equ	CTRL_LED	= 5		; Beleuchtung aus , wenn LOW
.equ	CTRL_PS1	= 6		; Prescaler :10 , LOW oder
.equ	CTRL_PS2	= 7		; Prescaler :64 , LOW

;* RAM Adressen
;---------------------------------------------------------------------------
.equ	ram_counter	= $0060		; Zhlerstnde , MSB:LSB,Timer-berlauf,Timer,Cnt2,Cnt1
.equ	ram_result	= $0070		; Ergebnis , ram_counter * prescale
.equ	ram_offset	= $0080		; Offset MSB:LSB , 4 Byte
.equ	ram_bcd		= $0090		; BCD Bytes
.equ	ram_uflag	= $00A0		; Flag Anzeige MHz/Hz
.equ	ram_prescale	= $00A1		; Verwendeter Vorteiler def. 1 
.equ	ram_bcdsw	= $00A2		; Eingestellter Wert am BCD Switch
.equ	ram_offtext	= $00A8		; Pointer auf Adresse OFFSetText
.equ	ram_ovflag	= $00AA		; Flag wenn Overflow, eintritt


; Reload / Delay f. Timer / Counter 1 Torzeit, 1 Sec. 
; 4 MHz / 256 = 15625
;---------------------------------------------------------------------------
.equ	TC_TIMSK	= 130		; Timer 0  / 1 Overflow
.equ	TC_TIM0C	= 7		; Timer 0  External Pin T0 rising edge

.equ	TC1_CNTL 	= $F7		; Reload 49911 $c2f7
.equ	TC1_CNTH	= $C2		; da 65536 - 15625 = 49911 Torzeit 1 sek

.equ	TC1_CK		= 4		; Teiler 256 
					; t = (65536-RELOAD)*TEILER * T_cyl 
					; T_cyl bei 4 MHz 250nS


;***** Interrupt vector table

reset:	rjmp	main			; Main rotine
	reti				; External Interrupt0 handle
	reti				; External Interrupt1 handle
	reti				; T/C1 Capture Event Interrupt handle
	reti				; T/C1 CompareA Interrupt handle
	reti				; T/C1 CompareB Interrupt handle
	rjmp	timer1_isr		; T/C1 Overflow Interrupt handle
	rjmp	timer0_isr		; T/C0 Overflow Interrupt handle
	reti				; SPI Transfer Complete Interrupt handle
	reti				; UART Rx Complete Interrupt handle
	reti				; UART Data Register Empty Interrupt handle
	reti				; UART Tx Complete Interrupt handle
	reti				; Analog Comparator Interrupt handle

;***** Functionen *********************************************************

;***************************************************************************
;*
;* InitRam 1.0,  DH1AAD / 05.12.2002 
;* InitRAM Initialisiert das RAM mit Startwerten
;*
;***************************************************************************
initRam:			
	ldi	temp,$00		; Standard 0.00 MHz
	sts	ram_ovflag,temp		; Overflow Anzeige aus
;	ldi	temp,$01		; zum Testen 28.350 MHz $01 b0 96 30
	sts	ram_counter+0,temp	; MSB berlauf
;	ldi	temp,$b0		; $60
	sts	ram_counter+1,temp	; Timer 0 
;	ldi	temp,$96		; $96
	sts	ram_counter+2,temp	; CNT2
;	ldi	temp,$30		; $30
	sts	ram_counter+3,temp	; LSB (CNT1)
	ldi	temp,CNT_PRE0		; Prescale 1 
	sts	ram_prescale,temp
	ret
; InitRAM - Ende
;***********************


;***************************************************************************
;*
;* Calc_result ,   DH1AAD / 05.12.2002 
;* Berechnet aus Overflow 
;* ( MSB*2^24 + Timer * 2^16 + Zhler2^8 + Zhler1 )* Vorteiler 
;* die Frequenz
;***************************************************************************
Calc_result:
	lds	mp32u0,ram_counter+3	; LSB
	lds	mp32u1,ram_counter+2	; 
	lds	mp32u2,ram_counter+1	; 
	lds	mp32u3,ram_counter+0	; MSB
	lds	mc32u0,ram_prescale	; LSB
	ldi	mc32u1,0		;
	ldi	mc32u2,0		;
	ldi	mc32u3,0		; Ergebnis steht in  
	rcall	MUL32			; R27:R26:R25:R24;R23:R22:R18:R19
	sts	ram_result+0,r27	; MSB
	sts	ram_result+1,r26
	sts	ram_result+2,r25
	sts	ram_result+3,r24
	sts	ram_result+4,r23
	sts	ram_result+5,r22
	sts	ram_result+6,r18
	sts	ram_result+7,r19	; LSB
	sbic	CTRL_PIN,CTRL_PTT	; PC4 auf LOW, dann Offset nicht bercksichtigen
	rcall	Calc_Offset
	ret				
;* Calc_result Ende

;***************************************************************************
;*
;* Calc_Offset 
;* Addiert Offset hinzu ,   DH1AAD / 05.12.2002 
;*
;***************************************************************************
;Ergebnis in R21:R20:R16:R17
Calc_Offset:
	ldi	temp,$00		; OV - Flag lschen
	sts	ram_ovflag,temp		
	lds 	add2l	,ram_offset+3	; Eingestellten Offset laden
	lds 	add2h	,ram_offset+2
	lds 	add2h2	,ram_offset+1
	lds 	add2h3	,ram_offset+0
	lds 	add1h3	,ram_result+4
	lds 	add1h2	,ram_result+5
	lds 	add1h	,ram_result+6
	lds 	add1l	,ram_result+7
	lds	r31	,ram_offset+4	; Flag , bei 1 Offset addieren sonst abziehen
	cpi	r31,0			; Sub ? 
	breq	co_sub			; Ja
	rcall	add32			; Nein, addieren 
	rjmp	cosv		
co_sub:	rcall	sub32
cosv:	cpi	R21,$FF			; Prfen ob MSB $FF, tritt auf wenn Fin < -Offset 
	breq	co_ns			; Werte dann nicht speichern.
	sts	ram_result+4,R21	; sonst, Werte speichern	
	sts	ram_result+5,R20
	sts	ram_result+6,R16
	sts	ram_result+7,R17
	rjmp	co_q
co_ns:	ldi	temp,$01		; OV - Flag setzen 
	sts	ram_ovflag,temp
co_q:	ret
; Calc_Offset Ende

;***************************************************************************
;*
;* Calc_BCD ,   DH1AAD / 05.12.2002 
;* Aus RAM BCD Codes erzeugen
;*
;***************************************************************************
Calc_BCD:
	lds	fbinmsb,ram_result+4	; MSB 
	lds	fbinHH,ram_result+5	; Es werden nur 4 Bytes verwendet , reicht  !
	lds	fbinH,ram_result+6	
	lds	fbinL,ram_result+7	; LSB
	rcall	bin2bcd32		; Wandlung nach BCD / Register R21-R18
	sts	ram_bcd+0,R15		; BCD Werte (2 / Byte)
	sts	ram_bcd+1,R14		; speichern 
	sts	ram_bcd+2,R13		; im RAM BCDRam
	sts	ram_bcd+3,R12
	sts	ram_bcd+4,R11		
	ret

;***************************************************************************
;*
;* port_setup	/ 7.12.02
;* Einstellen der DatenrichtungsRegister 
;*     
;***************************************************************************
port_setup:
	ldi	temp,$FF
	out	lcd_ddr,temp		; PortA , LCD Ausgang
	ldi	temp,$FE
	out	CNT_CDDR,temp		; PortB , Ausgang
	ldi	temp,0
	out	CNT_DDR,temp		; PortD , Zhler Eingang
	out	CTRL_DDR,temp		; PortC , Eingang diverses
	ret
; port_Setup Ende


;***************************************************************************
;*
;* port_check	/ 7.12.02
;* Liest ControlPort (PortC) 
;*
;***************************************************************************
port_check:
	ldi	temp,CNT_PRE0		; Default 
	sts	ram_prescale,temp	; speichern
	sbic	CTRL_PIN,CTRL_LED	; Low ? , 
	rjmp	pcn0			; 
	rcall	LCD_ledoff		; Nein , Beleuchtung aus
	rjmp	pcn1
pcn0:	rcall	LCD_ledon		; Beleuchtung ein
pcn1:	ldi	temp,CNT_PRE1		; Teilerverhltnis laden
	sbis	CTRL_PIN,CTRL_PS1	; Vorteiler 1 vorhanden ? 
	sts	ram_prescale,temp	; speichern	
	ldi	temp,CNT_PRE2		; Teilerverhltnis laden
	sbis	CTRL_PIN,CTRL_PS2	; Vorteiler 2 vorhanden ? 
	sts	ram_prescale,temp	; speichern
pcn2:	in	temp,CTRL_PIN		; ControlPort Pins einlesen
	ldi	r17,0b00001111		
	eor	temp,r17		; BCD , maskieren und invertieren
	cbr	temp,$F0		; oberes Nibble lschen
	sts	ram_bcdsw,temp		; und speichern
	ret
;* port_check Ende

;***************************************************************************
;*
;* timer1_setup	/ 8.12.02
;*
;***************************************************************************
timer1_setup:	ldi	temp,TC1_CNTL	; Initialize Reload Value
		out	TCNT1L,temp	; Counter vorbelegen
		ldi	temp,TC1_CNTH	; Initialize Reload Value
		out	TCNT1H,temp	; Counter vorbelegen
		ldi	temp,TC1_CK	; Teiler (5) = 1024 
		out	TCCR1B, temp	; Prescaler CK/1024 
		ret
;* timer1_setup Ende

;***************************************************************************
;*
;* Counter_Start
;*
;***************************************************************************
Counter_Start:
	sbi	CNT_CPORT,CNT_ENABLE	; Enable  auf High = aus
	cbi	CNT_CPORT,CNT_CLEAR	; Counter Reset,  auf LOW 
	nop				; warten... 
	nop				; bei 4 MHz 0,5S 
	nop				; warten... 
	sbi	CNT_CPORT,CNT_CLEAR	; Clear wieder auf high
	cbi	CNT_CPORT,CNT_Enable	; Enable auf low	
	sbi	CNT_CPORT,CNT_CNT0	; Output Enable auf high = disable
	sbi	CNT_CPORT,CNT_CNT1	; Output Enable auf high = disable
	sbi	CNT_CPORT,CNT_CLED	; Output high = LED aus
	ret
;* Counter_Start Ende

;***************************************************************************
;*
;* M A I N 
;*     
;***************************************************************************
main:	ldi	temp, LOW(RAMEND)	; StackPointer Setup
	out	SPL, temp		; Initialize SPL . 
	ldi	temp, HIGH(RAMEND)	; StackPointer Setup
	out	SPH, temp		; Initialize SPL . 

	ldi	temp,0b00000000		; INT1,INT0 mask, disabled
	out	GIMSK,temp
	bclr	I			; Global interrupt disable

	rcall	initRam			; RAM initialisieren

	rcall	port_setup		; Ports einstellen

	rcall	LCD_reset		; Display initialisieren
	rcall	LCD_clear		; Lschen
	rcall	LCD_cuoff		; Cursor aus
	rcall	LCD_ledon		; Led On

	rcall	port_check		; ControlPort einlesen 

	rcall	Calc_result		; Berechnung Frequenz
	rcall	Calc_BCD		; Umwandlung nach BCD

	ldi	r29,$02	
mss:	ldi	r28,$FF
	rcall	Show_Info_Line		; Anzeige 2 sec .
msh:	rcall	delay4ms		; Info Text
	dec	r28
	brne	msh
	dec	r29
	brne	mss
	rcall	LCD_clear		; Display Lschen

	rcall	Show_Freq		; Anzeige Frequenz
	rcall	ShowOffset		; Offset Berechnen und Anzeigen

	rcall	Counter_Start		; Start ...	
	rcall	timer1_setup		; Counter / Teiler einstellen

	ldi	temp,TC_TIM0C		; 
	out	TCCR0, temp		; External, T0, rising edge
	ldi	temp,TC_TIMSK		; Timer 1 Overflow Interrupt 128, Timer 0 Overflow 2
	out	TIMSK,temp 		; T/C0 Interrupt Enable

	sei				; Global interrupt enable
mainloop:
forever:rjmp mainloop			; Alles weitere nun durch ISR's

;***************************************************************************
;*
;* E N D E  -  M A I N 
;*     
;***************************************************************************

;***************************************************************************
;*
;* "timer0_isr"
;* DH1AAD  /  09.11.2002 
;*
;* Impulse zhlen , bei jedem Aufruf ram_counter+0 um 1 erhhen (16 MHz)
;***************************************************************************
timer0_isr:
;	bclr	I
;	ldi	temp,0b00000000 	; Clear general interrupt flag register
;	out	TIFR,temp	
;	out	TCCR0, temp		; Externl, T0, aus
;	in	ac_tmp,SREG		; temporarily store the Status register	


	lds	add1h,ram_counter+0	; MSB 16.777216 MHz 
	ldi	add1l,1
	add	add1h,add1l
	sts	ram_counter+0,add1h

;	ldi	temp,TC_TIM0C		; 
;	out	TCCR0, temp		; Externl, T0, falling Edge
;	out	SREG,ac_tmp		; restore Status register
	reti
;**** Ende timer0_isr **********


;***************************************************************************
;*
;* "timer1_isr"
;* DH1AAD  /  09.11.2002 
;*
;* Wird nach der Torzeit (1 Sekunde ) aufgerufen
;* Berechnet und Zeigt die aktuelle Frequenz an.
;***************************************************************************

timer1_isr:
	bclr	I			; Global interrupt disable
	in	ac_tmp,SREG		; temporarily store the Status register	

;	sbi	CNT_CPORT,CNT_ENABLE	; Counter stoppen, Enable auf high
					; Nicht aktiviert. Frequenzanzeige dann zu klein !?

	cbi	CNT_CPORT,CNT_CLED	; Output low = LED an

	ldi	temp,0b00000000 	; 
	out	TIFR,temp	
	out	TCCR0, temp		; Externl, T0, aus
	out	TCCR1B, temp		; Prescaler1 0 , Stop
	out	TIMSK,temp		
	out	TIFR,temp		; Clear general interrupt flag register

	lds	add1h,ram_counter+0	; MSB
	lds	add1l,ram_counter+1	; LSB
	ldi	add2h,0
	in	add2l,TCNT0		; TCNT0
	rcall	add16
	sts	ram_counter+0,add1h
	sts	ram_counter+1,add1l

	ldi	temp,$00		; 
	out	TCNT0,temp		; Timer0 auf 0 setzen	

	; Zhler einlesen
	cbi	CNT_CPORT,CNT_CNT1	; Output Enable auf low = enable
	rcall	delay40us
	in	r19,CNT_PIN
	sbi	CNT_CPORT,CNT_CNT1	; Output Enable auf high = disable
	sts	ram_counter+2,r19	; Internen Zhlerstand speichern

	cbi	CNT_CPORT,CNT_CNT0	; Output Enable auf low = enable
	rcall	delay40us
	in	r19,CNT_PIN
	sbi	CNT_CPORT,CNT_CNT0	; Output Enable auf high = disable
	sts	ram_counter+3,r19	; Internen Zhlerstand speichern

	cbi	CNT_CPORT,CNT_CLEAR	; Clear
	rcall	delay40us
	sbi	CNT_CPORT,CNT_CLEAR	; Clear aus

	rcall	port_check		; ControlPort einlesen 
	rcall	Calc_result		; Berechnung Frequenz
	rcall	Calc_BCD		; Umwandlung nach BCD
	rcall	Show_Freq		; Anzeige Frequenz
	rcall	ShowOffset		; Offset Berechnen und Anzeigen

	ldi	r16,$00
	sts	ram_counter+0,r16	; Ram auf 0 setzen
	sts	ram_counter+1,r16
	sts	ram_counter+2,r16
	sts	ram_counter+3,r16

	rcall	timer1_setup		; Teiler einstellen	
	ldi	temp,TC_TIMSK		; Timer 1 Overflow Interrupt
	out	TIMSK,temp 		; T/C0 Interrupt Enable

	ldi	temp,TC_TIM0C		; 
	out	TCCR0, temp		; Externl, T0, rising Edge
	out	SREG,ac_tmp		; restore Status register
	rcall	Counter_start		; Counter wieder starten 
	reti
;**** Ende timer0_isr **********

;***************************************************************************
;*
;* I N C L U D E S 
;*     
;***************************************************************************

.include "../Include/lcd4a.inc"		; Include's LCD 4 Bit - Mode ,DB = HighNibble
.include "../Include/math.inc"		; Include's Math
.include "fc_show.asm"			; Anzeige Funktionen

;***************************************************************************
;*
;* C O N S T A N T E N , C S E G 
;*     
;***************************************************************************
.CSEG
;.org xxxx ; An das Ende des Progs
;                                0123456789ABCDEF
        info1_txt:    	.db     "QRP-Zaehler 0.3 ",0x00
        info2_txt:    	.db     " DH1AAD 14.12.02",0x00
        hz_txt:    	.db     "Hz ",0x00
        mhz_txt:    	.db     "MHz",0x00
	offset_txt:	.db	"Offset:",0x00	

;* Offset Tabelle Aufbau: Offset MSB:LSB , Datenbytes 1,2, 3 ,4 
; Byte 5:  + wenn 1 , - wenn 0 
;* Offset Text der angezeigt werden soll

.equ	TAB	=$0800;			; Start der Datentabelle	
.equ	TAB_OFF	=$0005;			; Offset Bytes zu Text	
.org	TAB				; = WordAdr $01000 (4096)
;	                                   0123456789ABCDEF
	off0_text:	.db	$00,$00,$00,$00,$00,"  --- MHz",0x00
.org	TAB+$08
	off1_text:	.db	$00,$06,$F1,$58,$00," -455 KHz",0x00
.org	TAB+$10
	off2_text:	.db	$00,$3d,$09,$00,$00,"- 4.0 MHz",0x00
.org	TAB+$18
	off3_text:	.db	$00,$A3,$44,$E0,$00,"-10.7 MHz",0x00
.org	TAB+$20
	off4_text:	.db	$00,$89,$54,$40,$00," -9.0 MHz",0x00
.org	TAB+$28
	off5_text:	.db	$00,$00,$00,$00,$00,"  --- MHz",0x00
.org	TAB+$30
	off6_text:	.db	$00,$06,$F1,$58,$01," +455 KHz",0x00
.org	TAB+$38
	off7_text:	.db	$00,$3d,$09,$00,$01,"+ 4.0 MHz",0x00
.org	TAB+$40
	off8_text:	.db	$00,$89,$54,$40,$01,"+ 9.0 MHz",0x00
.org	TAB+$48
	off9_text:	.db	$00,$A3,$44,$E0,$01,"+10.7 MHz",0x00

;**** CSEG Daten Ende *************************